@@ -47,12 +47,12 @@ h2o.ls <- function() {
 #' Remove All Objects on the H2O Cluster
 #'
 #' Removes the data from the h2o cluster, but does not remove the local references.
-#' Retains frames and vectors specified in retained_elements argument.
-#' Retained keys must be keys of models and frames only. For models retained, training and validation frames are retained as well.
+#' Retains models, frames and vectors specified in retained_elements argument.
+#' Retained elements must be instances/ids of models and frames only. For models retained, training and validation frames are retained as well.
 #' Cross validation models of a retained model are NOT retained automatically, those must be specified explicitely.
 #'
 #' @param timeout_secs Timeout in seconds. Default is no timeout.
-#' @param retained_elements Frames and vectors to be retained. Other keys provided are ignored.
+#' @param retained_elements Instances or ids of models and frames to be retained. Combination of instances and ids in the same list is also a valid input.
 #' @seealso \code{\link{h2o.rm}}
 #' @examples
 #' \dontrun{
@@ -76,6 +76,10 @@ h2o.removeAll <- function(timeout_secs=0, retained_elements = c()) {
         retained_keys <- append(retained_keys, element@model_id)
       } else if (is.H2OFrame(element)) {
         retained_keys <- append(retained_keys, h2o.getId(element))
+      } else if( is.character(element) ) {
+        retained_keys <- append(retained_keys, element)
+      } else {
+        stop("The 'retained_elements' variable must be either an instance of H2OModel/H2OFrame or an id of H2OModel/H2OFrame.")
       }
     }
     
@@ -122,7 +126,7 @@ h2o.rm <- function(ids, cascade=TRUE) {
     } else if( is.character(xi) ) {
       .h2o.__remoteSend(paste0(.h2o.__DKV, "/",xi), method = "DELETE", .params=list(cascade=cascade))
     } else {
-      stop("input to h2o.rm must be a Keyed instance (e.g. H2OFrame, H2OModel) or character")
+      stop("Input to h2o.rm must be either an instance of H2OModel/H2OFrame or a character")
     }
   }
 
@@ -170,6 +174,28 @@ h2o.getFrame <- function(id) {
   fr
 }
 
+#' Get an list of all model ids present in the cluster
+#'
+#' @return Returns a vector of model ids.
+#' @examples
+#' \dontrun{
+#' library(h2o)
+#' h2o.init()
+#'
+#' iris_hf <- as.h2o(iris)
+#' model_id <- h2o.gbm(x = 1:4, y = 5, training_frame = iris_hf)@@model_id
+#' model_id_list <- h2o.list_models()
+#' }
+#' @export
+h2o.list_models <- function() {
+    models_json <- .h2o.__remoteSend(method = "GET", paste0(.h2o.__MODELS))$models
+    res <- NULL
+    for (json in models_json) {
+        res <- c(res, json$model_id$name)
+    }
+    res
+}
+
 #' Get an R reference to an H2O model
 #'
 #' Returns a reference to an existing model in the H2O instance.
@@ -272,15 +298,19 @@ h2o.getModel <- function(model_id) {
   # Convert ignored_columns/response_column to valid R x/y
 
 
-  parameters$x <- json$output$names
-  allparams$x  <- json$output$names
+  parameters$x <- if (is.null(json$output$original_names)) json$output$names else json$output$original_names
+  allparams$x  <- if (is.null(json$output$original_names)) json$output$names else json$output$original_names
     
   if (!is.null(parameters$response_column))
   {
     parameters$y <- parameters$response_column
     allparams$y <- allparams$response_column
-    parameters$x <- setdiff(parameters$x, parameters$y)
-    allparams$x <- setdiff(allparams$x, allparams$y)
+    .not_x <- function(params) {
+       c(params$y, params$ignored_columns, params$fold_column$column_name, params$weights_column$column_name,
+         params$offset_column$column_name)
+    }
+    parameters$x <- setdiff(parameters$x, .not_x(parameters))
+    allparams$x <- setdiff(allparams$x, .not_x(allparams))
   }
   allparams$ignored_columns <- NULL
   allparams$response_column <- NULL
@@ -516,6 +546,8 @@ h2o.download_mojo <- function(model, path=getwd(), get_genmodel_jar=FALSE, genmo
 #'
 #' @param model An H2OModel
 #' @param path The path where binary file should be downloaded. Downloaded to current directory by default.
+#' @param export_cross_validation_predictions A boolean flag indicating whether the download model should be
+#'      saved with CV Holdout Frame predictions. Default is not to export the predictions. 
 #'
 #' @examples
 #' \dontrun{
@@ -526,7 +558,7 @@ h2o.download_mojo <- function(model, path=getwd(), get_genmodel_jar=FALSE, genmo
 #' h2o.download_model(my_model)  # save to the current working directory
 #' }
 #' @export
-h2o.download_model <- function(model, path=NULL) {
+h2o.download_model <- function(model, path=NULL, export_cross_validation_predictions=FALSE) {
 
     if(!is.null(path) && !(is.character(path))){
       stop("The 'path' variable should be of type character")
@@ -537,6 +569,9 @@ h2o.download_model <- function(model, path=NULL) {
     if(is.null(path)){
       path = getwd()
     }
+    if(!is.logical(export_cross_validation_predictions)){
+      stop("The 'export_cross_validation_predictions' variable should be of type logical")
+    }
     
     #Get model id
     model_id <- model@model_id
@@ -547,7 +582,8 @@ h2o.download_model <- function(model, path=NULL) {
     
     #Path to save model, if `path` is provided
     file_path <- file.path(path, paste0(modelname))
-    writeBin(.h2o.doSafeGET(urlSuffix = urlSuffix, binary = TRUE), file_path, useBytes = TRUE)
+    parms <- list(export_cross_validation_predictions=export_cross_validation_predictions)
+    writeBin(.h2o.doSafeGET(urlSuffix = urlSuffix, binary = TRUE, parms = parms), file_path, useBytes = TRUE)
     
     return(paste0(file_path))
 }
