@@ -3,9 +3,9 @@
 # Then, it runs Deep Learning on the dataset
 # Note: This demo runs H2O on localhost:54321
 library(h2o)
-localH2O = h2o.init(ip = "localhost", port = 54321, startH2O = TRUE)
+h2o.init()
 
-prostate.hex = h2o.uploadFile(localH2O, path = system.file("extdata", "prostate.csv", package="h2o"), destination_frame = "prostate.hex")
+prostate.hex = h2o.uploadFile(path = system.file("extdata", "prostate.csv", package="h2o"), destination_frame = "prostate.hex")
 summary(prostate.hex)
 # Set the CAPSULE column to be a factor column then build model.
 prostate.hex$CAPSULE = as.factor(prostate.hex$CAPSULE)
